cellmove.mcmc <- function(sim,n.mcmc,mix=FALSE,boot=FALSE){

#
#  Mevin Hooten (20090606)
#
#  Fits animal movement on discrete spatial domain (grid) based on 
#  movement probabilities based on mixture of paths from one animal. 
#
#
#  ########
#  CHANGES:
#  ########
#
#  20090502:  Modified to work with GPS deer data.
#  20090503:  Modified 'get.grad' to use 'smooth.2d' instead of loess.
#  20090504:  Modifed from 'move.mix.mcmc.R' to use neg. binom. likelihood for residence time.
#  20090515:  Made changes to how it handles overdisperison parameter for Neg. Bin.
#  20090518:  Added calculation of likelihood weights for wait vs. move.
#  20090519:  Checked move portion of likelihood (which was always zero in previous version).
#  20090520:  Added features to help with automatic plotting of results.
#  20090523:  Modified from 'move.mix.NB.mcmc.R' to have conjugate updates.
#  20090527:  Re-implemented the mixture portion of the model.
#  20090529:  Modified from 'move.mix.G.mcmc.R' to do waits and moves in multinomial.
#  20090531:  Continued full multinomial modification.
#  20090606:  Modified from 'move.mix.MN.mcmc.R' to have 4 spatial fields.
#  20090608:  Added cats for loop in get.Q.
#  20090609:  Changed gamma1 to use x_j instead of x_i.
#  20100130:  Fixed s2=1.
#  20100203:  Modified from move.mix.MN.4.mcmc.R to have only 1 intercept for stays (none for stays).
#

###
###  Subroutines 
###

require(mvtnorm)
require(MCMCpack)
require(fields)

rtn1 <- function(mu,sig,a){
  u=runif(length(mu))
  y=mu-a+sig*qnorm(log(u)+pnorm((mu-a)/sig,log=TRUE),lower.tail=FALSE,log.p=TRUE) 
  y+a
}

rtn2 <- function(mu,sig,b){
  u=runif(length(mu))
  y=-mu+b+sig*qnorm(log(u)+pnorm((-mu+b)/sig,log=TRUE),lower.tail=FALSE,log.p=TRUE)
  -y+b
}

get.grid.grad <- function(mat){
  m=prod(dim(mat))
  dims=dim(mat)
  A.tilda.array=array(0,c(dims[1],dims[2],4))
  A.tilda.array[,2:dims[2],1]=mat[,1:(dims[2]-1)]-mat[,2:dims[2]]		# West
  A.tilda.array[2:dims[1],,2]=mat[1:(dims[1]-1),]-mat[2:dims[1],]		# North
  A.tilda.array[1:(dims[1]-1),,3]=mat[2:dims[1],]-mat[1:(dims[1]-1),]		# South
  A.tilda.array[,1:(dims[2]-1),4]=mat[,2:dims[2]]-mat[,1:(dims[2]-1)]		# East
  A.tilda=matrix(A.tilda.array,,4)
}

get.neigh.covs <- function(mat){
  m=prod(dim(mat))
  dims=dim(mat)
  A.tilda.array=array(0,c(dims[1],dims[2],4))
  A.tilda.array[,2:dims[2],1]=mat[,1:(dims[2]-1)]		# West
  A.tilda.array[2:dims[1],,2]=mat[1:(dims[1]-1),]		# North
  A.tilda.array[1:(dims[1]-1),,3]=mat[2:dims[1],]		# South
  A.tilda.array[,1:(dims[2]-1),4]=mat[,2:dims[2]]		# East
  A.tilda=matrix(A.tilda.array,,4)
}

get.grad <- function(mat,mat.idx){
  m=prod(dim(mat))
  dims=dim(mat)
  A.tilda.array=array(0,c(dims[1],dims[2],5))
  A.tilda.array[,2:dims[2],1]=mat[,1:(dims[2]-1)]-mat[,2:dims[2]]		# West
  A.tilda.array[2:dims[1],,2]=mat[1:(dims[1]-1),]-mat[2:dims[1],]		# North
  A.tilda.array[,,3]=mat							# No Move
  A.tilda.array[1:(dims[1]-1),,4]=mat[2:dims[1],]-mat[1:(dims[1]-1),]		# South
  A.tilda.array[,1:(dims[2]-1),5]=mat[,2:dims[2]]-mat[,1:(dims[2]-1)]		# East
  A.tilda=matrix(A.tilda.array,,5)
  Q.tilda=as.vector(t(A.tilda[mat.idx,]))
}

get.A <- function(mat){
  m=prod(dim(mat))
  dims=dim(mat)
  A.tilda.array=array(0,c(dims[1],dims[2],4))
  A.tilda.array[,2:dims[2],1]=mat[,1:(dims[2]-1)]-mat[,2:dims[2]]               # West
  A.tilda.array[2:dims[1],,2]=mat[1:(dims[1]-1),]-mat[2:dims[1],]               # North
  A.tilda.array[1:(dims[1]-1),,3]=mat[2:dims[1],]-mat[1:(dims[1]-1),]           # South
  A.tilda.array[,1:(dims[2]-1),4]=mat[,2:dims[2]]-mat[,1:(dims[2]-1)]           # East
  A.tilda=matrix(A.tilda.array,,4)
  A.tilda
}

dmultinom.2 <- function (x, size = NULL, prob, log = FALSE){ 
    K <- length(prob)
    if (length(x) != K) 
        stop("x[] and prob[] must be equal length vectors.")
    if (any(prob < 0) || (s <- sum(prob)) == 0) 
        stop("probabilities cannot be negative nor all 0.")
    prob <- prob/s
    x <- as.integer(x + 0.5)
    if (any(is.na(x))) 
        stop(return(NA))
    if (any(x < 0)) 
        stop("'x' must be non-negative")
    N <- sum(x)
    if (is.null(size)) 
        size <- N
    else if (size != N) 
        stop("size != sum(x), i.e. one is wrong")
    i0 <- prob == 0
    if (any(i0)) {
        if (any(x[i0] != 0)) 
            return(if (log) -Inf else 0)
        if (all(i0)) 
            return(if (log) 0 else 1)
        x <- x[!i0]
        prob <- prob[!i0]
    }
    r <- lgamma(size + 1) + sum(x * log(prob) - lgamma(x + 1))
    if (log) 
        r
    else exp(r)
}

###
###  Set Up Variables
###

X=sim$X
n.covs=dim(X)[2]
dims=sim$dims
n.ani=sim$n.ani

m=prod(dims)

n.burn=round(0.2*n.mcmc)
beta.save=matrix(NA,4*n.covs+1,n.mcmc)
z.path.save=matrix(NA,n.ani,n.mcmc)
theta.save=matrix(NA,n.ani,n.mcmc)
alpha1.mean=rep(0,m)
alpha2.mean=rep(0,m)
gamma1.mean=rep(0,m)
gamma2.mean=rep(0,m)

alpha1.idx=2:(n.covs+1)
alpha2.idx=(n.covs+2):(2*n.covs+1)
gamma1.idx=(2*n.covs+2):(3*n.covs+1)
gamma2.idx=(3*n.covs+2):(4*n.covs+1)

###
###  Get yit 
###

yit=sim$yd.list[[1]][,-1]
yit.vec=as.vector(t(yit))
yit.idx=sim$yd.list[[1]][,1]
n.obs=sapply(sim$yd.list,dim)[1,]
n.obs.min=min(n.obs)
n.yit=n.obs.min
z.path=rep(0,n.ani)
z.path[1]=1
path.numb=1

###
###  Setup Priors 
###

mu.beta=rep(0,4*n.covs+1)
theta=rep(1/n.ani,n.ani)
w=rep(.1,n.ani)
s2z=1

###
###  Get Directional Derivatives 
###

cat("Getting Q ...","\n")
Q.grid.array=array(0,c(m,5,n.covs))
Q.g1.array=array(0,c(m,5,n.covs))
for(j in 1:n.covs){
  Q.grid.array[,-3,j]=get.grid.grad(matrix(X[,j],dims[1],dims[2]))
  Q.g1.array[,-3,j]=get.neigh.covs(matrix(X[,j],dims[1],dims[2]))
}
Q.g2.sum=apply(Q.grid.array,c(1,3),sum)

Q.list=vector("list",n.ani)
Z.list=vector("list",n.ani)

for(i in 1:n.ani){
  cat(i," ")
  z.path=rep(0,n.ani)
  z.path[i]=1
  yit=sim$yd.list[[i]][,-1]
  w.idx=matrix(FALSE,dim(yit)[1],dim(yit)[2])
  w.idx[,3]=TRUE
  w.idx=as.vector(t(w.idx))
  yit.vec=as.vector(t(yit))
  yit.idx=sim$yd.list[[i]][,1]
  yit.locs=sim$lc.locs[sim$yd.list[[i]][,1],]

  Q.g2=matrix(0,length(yit.vec),n.covs)
  Q.g1=matrix(0,length(yit.vec),n.covs)
  for(j in 1:n.covs){
    Q.g2[,j]=as.vector(t(Q.grid.array[yit.idx,,j]))
    Q.g1[,j]=as.vector(t(Q.g1.array[yit.idx,,j]))
  } 

  Q=matrix(1,length(yit.vec),4*n.covs+1)
  Q[,alpha1.idx]=matrix(rep(X[yit.idx,],5),,n.covs)
  Q[,alpha2.idx]=matrix(rep(Q.g2.sum[yit.idx,],5),,n.covs)
  Q[,gamma1.idx]=Q.g1
  Q[,gamma2.idx]=Q.g2

  Q[!w.idx,1:(2*n.covs+1)]=0
  Q[w.idx,(2*n.covs+2):(4*n.covs+1)]=0

  Q.list[[i]]=Q

  if(i==1){
    beta=as.vector(coef(glm(yit.vec~0+Q,family=binomial)))
    Sig.beta=100*solve(t(Q)%*%Q)
  }

  Qbeta=Q%*%beta

  Z.mat=matrix(0,5,n.yit)
  Qbeta.mat=matrix(Qbeta,5,n.yit)

  Y.idx=(t(yit)==1)
  Z.mat[Y.idx]=3
  Z.mat[!Y.idx]=rtn2(Qbeta.mat[!Y.idx],sqrt(s2z),rep(Z.mat[Y.idx],each=4))
  Z.mat[Y.idx]=rtn1(Qbeta.mat[Y.idx],sqrt(s2z),apply(matrix(Z.mat[!Y.idx],4,),2,max))
  Z=as.vector(Z.mat)
  Z.list[[i]]=Z
}
cat("\n")


####
####  Initialize Timing Variables 
####

time1=proc.time()
time2=time1
timeidx=0

###
###  Begin MCMC Loop 
###

cat("MCMC Loop ...","\n")
for(k in 1:n.mcmc){
  cat(k," ")

  ###
  ### Timing Calculations
  ###
 
  if(k==12){
    tentime=(proc.time()-time1)[3]
    cat("\n",tentime*(n.mcmc/600),"expected minutes","\n")
  }
  if(k%%100==0){
    timeidx=timeidx+1
    elapsetime=(proc.time()-time2)[3]
    cat("\n",elapsetime/60,"elapsed minutes"," ")
    leftidx=n.mcmc/100 - timeidx
    cat(" ",(elapsetime/timeidx)*leftidx/60,"remaining minutes","\n")
  }

  ###
  ###  Sample z.path 
  ###

  if(mix){
    lik.vec=rep(0,n.ani)
    for(j in 1:n.ani){
      yit=sim$yd.list[[j]][,-1]
      tmp.lik=0  
      Qbeta=Q.list[[j]]%*%beta
      tmp.lik=sum(dnorm(Z.list[[j]],Qbeta,sqrt(s2z),log=TRUE),na.rm=TRUE)
      lik.vec[j]=tmp.lik+log(theta[j]) 
    }
    theta.tmp=rep(0,n.ani)
    for(j in 1:n.ani){
      theta.tmp[j]=1/sum(exp(lik.vec-lik.vec[j])) 
    }

    z.path=as.vector(rmultinom(1,1,theta.tmp))
    path.numb=(1:n.ani)[z.path==1]

    cat(path.numb," ")
    yit.idx=sim$yd.list[[path.numb]][,1]
    Q=Q.list[[path.numb]]
    Qbeta=Q%*%beta
    yit=sim$yd.list[[path.numb]][,-1]
    Y.idx=(t(yit)==1)
    yit.locs=sim$lc.locs[sim$yd.list[[path.numb]][,1],]
    Z=Z.list[[path.numb]]

    ###
    ###  Sample theta 
    ###

    theta=as.vector(rdirichlet(1,w+z.path))

  }

  ###
  ###  Bootstrap Paths 
  ###
   
  if(boot){
    path.numb=sample(1:n.ani,1)
    yit.idx=sim$yd.list[[path.numb]][,1]
    Q=Q.list[[path.numb]]
    Qbeta=Q%*%beta
    yit=sim$yd.list[[path.numb]][,-1]
    Y.idx=(t(yit)==1)
    yit.locs=sim$lc.locs[sim$yd.list[[path.numb]][,1],]
    Z=Z.list[[path.numb]]
  }

  ###
  ###  Sample Z 
  ###

  Z.mat=matrix(Z,5,n.yit)
  Qbeta.mat=matrix(Qbeta,5,n.yit)
  Z.mat[!Y.idx]=rtn2(Qbeta.mat[!Y.idx],sqrt(s2z),rep(Z.mat[Y.idx],each=4))
  Z.mat[Y.idx]=rtn1(Qbeta.mat[Y.idx],sqrt(s2z),apply(matrix(Z.mat[!Y.idx],4,),2,max))
  Z=as.vector(Z.mat)
  Z.list[[path.numb]]=Z

  if(mix | boot){
    for(i in (1:n.ani)[-path.numb]){
      z.path.tmp=rep(0,n.ani)
      z.path.tmp[i]=1
      yit.tmp=sim$yd.list[[i]][,-1]
      yit.vec.tmp=as.vector(t(yit.tmp))
      yit.idx.tmp=sim$yd.list[[i]][,1]
      yit.locs.tmp=sim$lc.locs[sim$yd.list[[i]][,1],]

      Qbeta.tmp=Q.list[[i]]%*%beta
      Z.mat.tmp=matrix(Z.list[[i]],5,n.yit)
      Qbeta.mat.tmp=matrix(Qbeta.tmp,5,n.yit)

      Y.idx.tmp=(t(yit.tmp)==1)
      Z.mat.tmp[!Y.idx.tmp]=rtn2(Qbeta.mat.tmp[!Y.idx.tmp],sqrt(s2z),rep(Z.mat.tmp[Y.idx.tmp],each=4))
      Z.mat.tmp[Y.idx.tmp]=rtn1(Qbeta.mat.tmp[Y.idx.tmp],sqrt(s2z),apply(matrix(Z.mat.tmp[!Y.idx.tmp],4,),2,max))
      Z.tmp=as.vector(Z.mat.tmp)
      Z.list[[i]]=Z.tmp
    }
  }


  ###
  ###  Sample Beta
  ###

  tmpvar=solve(t(Q)%*%Q/s2z + solve(Sig.beta))
  tmpmean=tmpvar%*%t(t(Z)%*%Q/s2z + t(mu.beta)%*%solve(Sig.beta))
  beta=as.vector(rmvnorm(1,tmpmean,tmpvar,method="svd"))

  ###
  ###  Get alphas, gammas, and Qbeta 
  ###

  alpha1=X%*%beta[alpha1.idx]
  alpha2=Q.g2.sum%*%beta[alpha2.idx]
  gamma1=X%*%beta[gamma1.idx]
  gamma2=X%*%beta[gamma2.idx]
  Qbeta=Q%*%beta

  ###
  ###  Save Samples 
  ###

  beta.save[,k]=beta
  theta.save[,k]=theta
  z.path.save[,k]=z.path
  if(k>n.burn){
    alpha1.mean=alpha1.mean+alpha1/(n.mcmc-n.burn)
    alpha2.mean=alpha2.mean+alpha2/(n.mcmc-n.burn)
    gamma1.mean=gamma1.mean+gamma1/(n.mcmc-n.burn)
    gamma2.mean=gamma2.mean+gamma2/(n.mcmc-n.burn)
  }
  if(k%%50==0){
    tmp.out=list(beta.save=beta.save,n.mcmc=n.mcmc,theta.save=theta.save,n.burn=n.burn,dims=dims,k=k,alpha1.mean=alpha1.mean,alpha2.mean=alpha2.mean,gamma1.mean=gamma1.mean,gamma2.mean=gamma2.mean)
    save(tmp.out,file="mcmc.out.RData")
  }
  cat("\n")
}
cat("\n")

###
###  End MCMC Loop 
###


###
###  Write Output 
###

list(beta.save=beta.save,n.mcmc=n.mcmc,theta.save=theta.save,z.path.save=z.path.save,n.burn=n.burn,dims=dims,X=X,dims=dims,x=sim$x,y=sim$y,Q=Q,yit=yit,yit.locs=yit.locs,n.yit=n.yit,alpha1.mean=alpha1.mean,alpha2.mean=alpha2.mean,gamma1.mean=gamma1.mean,gamma2.mean=gamma2.mean,alpha1.idx=alpha1.idx,alpha2.idx=alpha2.idx,gamma1.idx=gamma1.idx,gamma2.idx=gamma2.idx)

}
